import sys
import argparse
from maudio import get_cipher,get_audio

def main():
    parser = argparse.ArgumentParser(description="maudio is a command-line tool that converts text to morse code and can generate audio output file (WAV format).",prog="maudio", usage="%(prog)s \"message\" [options]")

    if sys.stdin.isatty():
        parser.add_argument("message",type=str,help="Message to convert to morse")

    parser.add_argument("-v","--verbose",action="store_true",help="Enable verbose output")
    parser.add_argument("-o","--output",type=str,default="temp.wav",metavar="",help="Output WAV file name (default: \'temp.wav\')")
    parser.add_argument("-f","--frequency",type=int,default=600,metavar="",help="Tone frequency in Hz (default: 600)")
    parser.add_argument("-s","--sample-rate",type=int,default=44100,metavar="",help="Audio sample rate in Hz (default: 44100)")
    parser.add_argument("-b","--bits",type=int,default=16,metavar="",help="Bit depth (default: 16)")
    parser.add_argument("-w","--wpm",type=int,default=18,metavar="",help="Words per minute (speed) (default: 18)")
    parser.add_argument("-a","--amplitude",type=float,default=0.5,metavar="",help="Tone amplitude (0.0 to 1.0) (default: 0.5)")
    parser.add_argument("--noaudio",action="store_true",help="Prints Morse code without generating audio.")
    parser.add_argument("--farns",type=int,metavar="",help="Apply Farnsworth timing with given WPM for spacing")

    args=parser.parse_args()

    message = args.message if hasattr(args,"message") else sys.stdin.read()

    args.farns_desc = f"{args.farns} WPM" if args.farns else "disabled"

    if args.noaudio:
        cipher = get_cipher(message)
        print(cipher if not args.verbose else f"\ncipher : {cipher}")
        sys.exit(0)

    try:
        if args.verbose:
            print("Audio Encoding Settings\n")

            args_dict = {
                "frequency":   args.frequency,
                "sample_rate": args.sample_rate,
                "bits":        args.bits,
                "wpm":         args.wpm,
                "amplitude":   args.amplitude,
                "farns":       args.farns_desc,
            }

            for k, v in args_dict.items():
                print(f"{k:12}: {v}")

        print(f"{'output_file':12}: {args.output}")
        time_elapsed = get_audio( get_cipher(message) , args.output , args.wpm , args.frequency , bits=args.bits , rate = args.sample_rate , amp=args.amplitude , farns=args.farns )
        print("\nTime elapsed: {}ms".format(time_elapsed))
    except Exception as e:
        print(f"Error: {e}")
        sys.exit(1)


if __name__ == "__main__":
    main()



import time
import wave,struct
from .MorseTable import forwardTable

def get_cipher(message):
    cipher = ''
    for letter in " ".join(message.upper().split()):
        if letter != ' ':
            try:
                cipher += forwardTable[letter] + ' '
            except KeyError:
                pass
        else:
            cipher += ' '
    return cipher

def get_audio( cipher , filename , wpm , frequency=600 , **kwargs):

    start = time.time()

    CHANNEL_NO = 1

    #Filter kwargs set to None
    kwargs = {k: v for k, v in kwargs.items() if v}

    for key in kwargs:
        if key not in [ "farns" , "amp" , "bits" , "rate" ]:
            raise TypeError("Unknown argument \"{}\"".format(key))

    WPM_MIN = 5
    WPM_MAX = 100
    FREQ_MIN = 600
    FREQ_MAX = 1000
    RATE_MIN = 2000
    RATE_MAX = 44100
    BIT_VALUES = [ 8 , 16 , 32 ]

    FARNS = kwargs.pop("farns",wpm)
    BITS = kwargs.pop("bits",16)
    RATE = kwargs.pop("rate",4410)
    AMP = kwargs.pop("amp",0.5)

    if not ( WPM_MIN <= wpm <= WPM_MAX and isinstance(wpm,int) ):
        raise Exception("wpm should be and integer value between {} and {}".format(WPM_MIN,WPM_MAX))

    if not ( WPM_MIN <= FARNS <= wpm and isinstance(FARNS,int) ):
        raise Exception("farns should be and integer value less than or equal to wpm and in the range ( {} - {} )".format(WPM_MIN,WPM_MAX))

    if len(filename) < 5 or filename[-4:] != '.wav':
        filename +='.wav'

    if not (FREQ_MIN <= frequency <= FREQ_MAX and isinstance(frequency,int) ):
        raise Exception("frequency should be an integer value between {} and {} (in Hz)".format(FREQ_MIN,FREQ_MAX))

    if BITS not in BIT_VALUES:
        raise Exception("bits : Unsupported bit value \"{}\"".format(BITS))

    if not ( RATE_MIN <= RATE <= RATE_MAX and isinstance(RATE,int) ):
        raise Exception("rate(sample rate) should be and integer value between {} and {}".format(RATE_MIN,RATE_MAX))

    if not ( 0 <= AMP <= 1 and ( isinstance(AMP,int) or isinstance(AMP,float) )):
        raise Exception("amp should be a value between 0 and 1")

    if not all(x in ('-', '.' ,' ' ) for x in cipher):
        raise Exception("cipher should only contain fullstops(dits) ,hyphens(dah) and spaces")

    dit_dur = float(60 / (50 * wpm))
    fdit_dur = (60 / FARNS - 31 * dit_dur) / 19

    period  = int( RATE / frequency )
    sqr_sample = period * [0]

    for i in range(period):
        if i < period/2:
            sqr_sample[i] = 1
        else:
            sqr_sample[i] = -1

    max_amplitude = float((int((2 ** BITS) / 2) - 1) * AMP)

    nframes = RATE / period    #frames per second
    sqr_bin = b''.join(struct.pack('h', int(i * max_amplitude)) for i in sqr_sample)
    dit = sqr_bin * int( nframes * dit_dur )
    dah = sqr_bin * int( nframes * dit_dur * 3)

    zero = struct.pack('h',0 )
    intra_char = zero * period * int( nframes * dit_dur )
    inter_char = zero * period * int( nframes * fdit_dur * 3 )
    word_space = zero * period * int( nframes * fdit_dur * 7 )

    bins = [ dit , dah , intra_char , inter_char , word_space ]

    w = wave.open(filename, 'wb')
    w.setnchannels (CHANNEL_NO ) # Mono
    w.setsampwidth( int(BITS / 8) ) # Sample is 1 Bytes
    w.setframerate( RATE ) # Sampling Frequency
    w.setcomptype('NONE','Not Compressed')

    waveform = []

    count = 0
    for signal in cipher:
        if signal == '-':
            waveform += [1]
            if cipher[count + 1] != ' ':
                    waveform += [2]
        elif signal == '.':
            waveform += [0]
            if cipher[count + 1] != ' ':
                waveform += [2]
        elif signal == ' ' and cipher[count + 1] != ' ':
            waveform += [3]
        elif signal == ' ' and cipher[count + 1] == ' ':
            waveform += [4]
        else:
            #Do nothing
            pass
        count += 1
        if count == (len(cipher) -1):
            waveform += [3]
            break

    #divide_chunks
    chunk_size = 5
    chunks = [waveform[i * chunk_size:(i + 1) * chunk_size] for i in range((len(waveform) + chunk_size - 1) // chunk_size )]


    for chunk in chunks:
        frames = b''.join(bins[sample] for sample in chunk)
        w.writeframesraw(frames)

    try:
        w.close()
    except:
        raise RuntimeError("wave.close() error , failed to write wav file during execution")

    end = time.time()
    return round((end - start) *1000,2)



from setuptools import setup,find_packages

from pathlib import Path
this_directory = Path(__file__).parent
long_description = Path("README.md").read_text(encoding="utf-8").replace("<pre>", "# Maudio - Minimal Morse code audio encoder\n").split("</pre>")[-1].lstrip()

giturl="https://github.com/Mohd-Sinan/maudio"

setup(
      name='maudio',
      version='0.0.1',
      author='Mohammed Sinan KH',
      maintainer='Mohammed Sinan KH',
      author_email='devsinankh123@gmail.com',
      license='MIT',
      description='Minimal Morse code audio encoder ( WAV format )',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url=giturl,
      packages=[ 'maudio' ,],

      classifiers=[
      'Development Status :: 3 - Alpha',
      'License :: OSI Approved :: MIT License',
      'Intended Audience :: Developers',
      'Topic :: Multimedia :: Sound/Audio',
      'Programming Language :: Python :: 3.2',
      'Programming Language :: Python :: 3.6',
      'Programming Language :: Python :: 3.7',
      'Programming Language :: Python :: 3.8',
      'Programming Language :: Python :: 3.9',
      ],
      keywords='morse code audio generation',
      python_requires='>=3.6',

      entry_points={
        'console_scripts': [
        'maudio=maudio.__main__:main',
          ],
      },
      project_urls={
        'Source': giturl,
        'Tracker': giturl +'/issues',
      },
)
